#!/usr/bin/php
<?php require_once('EasyBackup.inc'); ?>
<?php require_once('Lib_Cron.inc');   ?>
<?php require_once('Lib_Gnome.inc');  ?>
<?php require_once('Lib_Others.inc'); ?>
<?php

$IsTest   = in_array("-t", $argv);
$IsToWait = in_array("-w", $argv);

// Processs Option ----------------------------------------------------------------------------------------------------
if($argc > 1) {
	if(in_array("--test", $argv) || in_array("-t", $argv)) $IsTest = true;
	if(in_array("--help", $argv) || in_array("-h", $argv)) showHelpMessage();

	// Often used option
	$Is_u = in_array("-u", $argv);
	$Is_I = in_array("-I", $argv);
	$Is_i = in_array("-i", $argv);
	$Is_l = in_array("-l", $argv);

	$Is_o = $Configs["OpenTerminal"];
	if(in_array("-o", $argv)) {
		for($i = 1;  $i < $argc; $i++) {
			if($argv[$i] != "-o") continue;
			if($i == $argc) { echo "Invalid parameter.\n"; exit(0); }
			$Is_o = ($argv[$i + 1] == "yes");
		}
	}

	// Remove the crontab job
	if($Is_u || $Is_i || $Is_I) removeEasyBackupCrontabJop(/* ToExitWhenDone */ !($Is_i || $Is_I));
	
	if($Is_i || $Is_I) {
		// Cannot install both kind of crontab at the same time
		if($Is_i && $Is_I) { echo "Invalid parameter!!! -i and -I cannot be used together."; exit(0); }

		// Loop to find the time format value
		for($i = 1;  $i < $argc; $i++) {
			// Screen for -i or -I
			if(($argv[$i] != "-i") && ($argv[$i] != "-I")) continue;
			// Check if no value
			if($i == $argc) { echo "Invalid parameter.\n"; exit(0); }

			// Get the crontab job time format
			$Time     = $argv[$i + 1];
			$CronTime = $Is_i?getCronjobTimeForEvery($Time):getCronjobTimeForNext($Time);

			// Check if the time is well-formed
			if($CronTime == null) { echo "Invalid date/time format: $Time\n"; exit(0); }

			// Install the next cron tab time
			$ExtraOption = "";
			if($Is_I)     $ExtraOption .= " -I ".$Time." ";
			if($IsToWait) $ExtraOption .= " -w ";

			// Install the cron job
			installEasyBackupCrontabJob($CronTime, $ExtraOption);
			break;
		}
		// -i will not continue the backup
		if($Is_i) exit(0);
	}
	if($Is_l || $Is_o) {
		// Make the AppName a full path
		$AppName = $argv[0]; if(!strStartWith("/", $AppName)) $AppName = getcwd()."/".$AppName;
		// Get the real path so that there is no need to worry about different path via symbol-link.
		$AppName = getRealPath($AppName);
		// Add the config file suffix
		$AppName .= ($IsNonDefaultConfig?"":" --ConfigFile ".$ConfigFileName)." -o no ";

		if($Is_o) {
			$AppName = $Display."gnome-terminal -e \"".$AppName.($Is_l?" -l ":"").($IsToWait?" -w ":"")."\"";
			echo trim(shell_exec($AppName));
		} else {
			// Prepare the log file
			$LogName = 'Backup-'.($IsNonDefaultConfig?"":$ConfigFileName).'-{'.date("Y-m[M]-d--H:i").'}.log';
			$LogFile = $HomeDir.'.'.$Program.'/logs/'.$LogName;

			if($Is_l) $AppName .= ' | tee "'.$LogFile.'"';

			// Perform the backup into the log file
			popen($AppName, 'w');

			// See if the execution return problem and then copy the log file to Desktop so that user is notified
			if($Is_l) {
				if(!file_exists($LogFile)) {
					showInfoGUI("Problem?", "There is some problem in the backup process. The log file is missing.", 600, 300, 10);
				} else if(
					(null != trim(shell_Exec('cat "'.$LogFile.'" | grep -o "Traceback (most recent call last):"'))) || 
				    (null != trim(shell_Exec('cat "'.$LogFile.'" | grep -o -E "^Exception .*[eE]rror.*:$" | grep -v "/home/.*/.gvfs"')))) {
					shell_exec('cp "'.$LogFile.'" '.$HomeDir.'Desktop/'.$LogName.'.error');
					showInfoGUI("Problem?", "There is some error(s) in the backup process.", 600, 300, 10);
				} else if((null != trim(shell_Exec('cat "'.$LogFile.'" | grep -o "Other instance of rdiff-backup is running. Please try again later."')))) {
					
				}
			}
		}
		exit(0);
	}
}

// Obtains the list of all files with related emblems -----------------------------------------------------------------
$Includes = getListsFromSystemEmblems($UserName, $Configs["EmplemsToIncluded"]);
$Excludes = getListsFromSystemEmblems($UserName, $Configs["EmplemsToExcluded"]);
$ExIns    = combineExcludesIncludes($Excludes, $Includes);
if($IsTest) {
	echo "Files/Folders marked as included: ";       print_r($Includes);
	echo "Files/Folders marked as excluded: ";       print_r($Excludes);
	echo "Combine list of included and exclulded: "; print_r($ExIns);
}
unset($Includes);
unset($Excludes);

// Prepare the RDiff command prefix based on the configuration file
$Prefix = getCommandPrefixFromConfig();

if($Configs["PreBackupCommand"] != null) {
	echo "\n\n\n";
	echo "=========================================================================================================\n";
	echo 'Executing PreBackupCommand:'."\n\t".$Configs["PreBackupCommand"]."\n";
	$Result = trim(shell_exec($Configs["PreBackupCommand"]));
	if(strStartWith("EVALME: ", $Result)) {
		$Result = substr($Result, strlen("EVALME: ")).";";
		echo "---------------------------------------------------------------------------------------------------------\n";
		echo 'Evaling the return of PreBackupCommand:'."\n\t".$Result."\n";
		echo "---------------------------------------------------------------------------------------------------------\n";
		eval($Result);
	}
	echo "=========================================================================================================\n";
	echo "\n\n\n";
}

// Preform backup
$BackupCmd = join(" \\\n    ", $Prefix)."\\\n    ".join("\\\n    ", $ExIns)." / '".$BackupDIR."'";
echo "=========================================================================================================\n";
echo "Command:\n  ".$BackupCmd."\n";
echo "---------------------------------------------------------------------------------------------------------\n";
if($IsTest) {
	echo "NO BACKUP!!! This is a test (-t).\n";
} else {
	// The target backup directory
	$BackupCmd = join(" ", $Prefix)." ".join(" ", $ExIns)." / '".$BackupDIR."'";
	$BackupCmd = ($Configs["BackupAsRoot"]?'gksudo "':'').$BackupCmd.($Configs["BackupAsRoot"]?'"':'');
	EnsureNoRdiffBackupRunning();
	echo "Command:\n  ".$BackupCmd."\n";
	echo "\n\n";
	popen($BackupCmd, 'w');
	echo "\n\n";
	// Change the permission of the backup folder to the user
	if($Configs["BackupAsRoot"]) {
		$BackupCmd = $Display."gksudo \"chown -Rf root:".$UserName." '".$BackupDIR."'\""; echo "Command:\n  ".$BackupCmd."\n"; echo shell_exec($BackupCmd)."\n";
		$BackupCmd = $Display."gksudo \"chmod -Rf 750"            ." '".$BackupDIR."'\""; echo "Command:\n  ".$BackupCmd."\n"; echo shell_exec($BackupCmd)."\n";
	}

	if($Configs["RemoveOlderThan"] != null) {
		// Remove the old snapshorts
		$BackupCmd = 
		        ($Configs["BackupAsRoot"]?'gksudo "':'').
		        "rdiff-backup --force --remove-older-than '".$Configs["RemoveOlderThan"]."' '".
		        $BackupDIR."'".($Configs["BackupAsRoot"]?'"':'').
		        " | grep -v -E 'No increments older than .* found, exiting.'";

		$RemoveResult = trim(shell_exec($BackupCmd));
		if($RemoveResult != null) echo "Remove old snapshot: \n".'    '.$RemoveResult;
	}
}

if($Configs["PostBackupCommand"] != null) {
	echo "\n\n\n";
	echo "=========================================================================================================\n";
	echo 'Executing PostBackupCommand:'."\n\t".$Configs["PostBackupCommand"]."\n";
	$Result = trim(shell_exec($Configs["PostBackupCommand"]));
	if(strStartWith("EVALME: ", $Result)) {
		$Result = substr($Result, strlen("EVALME: ")).";";
		echo "---------------------------------------------------------------------------------------------------------\n";
		echo 'Evaling the return of PostBackupCommand:'."\n\t".$Result."\n";
		echo "---------------------------------------------------------------------------------------------------------\n";
		eval($Result);
	}
	echo "=========================================================================================================\n";
	echo "\n\n\n";
}

// Done
echo "Done!!! - Hopefully it success.\n";
echo "Finish at at ".date("Y-m[M]-d--H:i")."\n";

// Functions -----------------------------------------------------------------------------------------------------------

/** Display help message */
function showHelpMessage() {
	global $ProgramFull;
?>
<?php echo $ProgramFull."\n\n"; ?>
This command is to backup files in folder marked by gnome emblems using 'rdiff-back'. Only folders marks with
    'ToBackup' emblems and all its content will be backup unless 'ToNotBackup' emblems is used. For the moment, only
    marked folders in the current user home folder are supported.

Example: ---------------------------------------------------------------------------------------------------------------
	Emblem			Folder
	<ToBackup>		/home/user/Documents
					/home/user/Documents/Desktop
	<ToNotBackup>	/home/user/Documents/Desktop/Junks

In this case, All files/folders inside '/home/user/Documents' except the files and folders in
	'/home/user/Documents/Desktop/Junks' will be backed up. In other words, Shell command similar
	the following will be executed.

	rdiff-backup --exclude "/home/user/Documents/Desktop/Junks" /home/user/Documents 

If both <ToBackup> and <ToNotBackup> is applied to the same folder, the folder will not be back up.
------------------------------------------------------------------------------------------------------------------------

Usage:
    <?php echo $ProgramFull; ?> <option>

    <option>
    --help, -h:
	Show this message

    -t:
	Just simulate and not actually do the backup

	--ConfigFile:
	Select a configuration file

    -i <time string>:
    Set automatic backup
	
    The <time string> can be as the following:
        A day in a week:
            (sun|mon|tue|wed|thu|fri|sat)
            Examples:
                sun = backup every sunday (mid night of sunday)
   
        Every certail amount of time or 'every-x' time:
            <Step><Unit> as [0-9]+[mhDM] or [0-9]+(min|hour|day|month)
            For Unit,
       	        m = Minute
       	        h = Hour
       	        D = Day
       	        M = Month
            Examples:
                10m = every 10 minutes
                12h = every 12 hours
                14D = every 14 days
  
        A valid cronjob time:
            "0 0 1 * *" = For at the 1st midnight of every month

	This option will set the crontab job schedule for exactly given period. If at that particular moment, the computer
	    this one on, the backup media is not ready or even another rdiff is running, the operation will be skipped.
	For example: if "EasyBackup -i 1D" is run at 10:00am if Sep 1, 2008, the cron job schedule will be set to run at
	    10:00am of every day from Sep 1, 2008 and on.
	For example: if "EasyBackup -i sun" is run at 10:00am if Sep 1, 2008, the cron job schedule will be set to run at
	    10:00am of every sunday from Sep 1, 2008 and on (Sep 7, 2008 and every sunday from then).

	Setting automatic backup this way should only be used for the short-time backup policy (like, within a few days) as
	   a skipping of a backup will not cause too much unbackuped files/folders.

    -I <time string>:
    Set automatic backup and ensure that it will not be skipped.

    The <time string> can be as the following:
        Every certail amount of time or 'every-x' time:
            <Step><Unit> as [0-9]+[mhDM] or [0-9]+(min|hour|day|week|month) or [0-9]+(mins|hours|days|weeks|months)
            For Unit,
       	        m = Minute
       	        h = Hour
       	        D = Day
       	        W = Week
       	        M = Month (no more than 10 month)
            Examples:
                10m = every 10 minutes
                12h = every 12 hours
                14D = every 14 days

	This option is done by set the cron job for every minutes after the given period of time. When the operation is
	    called by cron to perform, it will remove the schedule and reschedule for the same period of time for the next
	    operation.
	For example: if "EasyBackup -I 2W" is run at 10:00am if Sep 1, 2008, the cron job schedule will be set to run for
        every minute after 10:01am of Sep 15, 2009. This way, even if the computer is not run at 10:01am on that day.
        the backup will be performed as soon as the computer is started.
        Once, EasyBackup is run by the cron tab. it will remove the schedule, reschedule and perform the operation.
	    The reschedule will be for the next 2W. Let's say if the computer is off at 10:01am of Sep 15, 2008 but is
	    turned on at 4:35pm of the next day (Sep 16, 2008). The reschedule will be for 4:36pm of Sep 30, 2008.

	This automatic setup method is suggested. Especially for a long-period backup (A week or longer).

    -u:
    Remove the automatic backup

    -l:
    Produce a log of the backup in "~/.EasyHomeBackup/logs/"

<?php
	exit(0);
}

/** Install a schedule backup */
function installEasyBackupCrontabJob($CronTime, $ExtraOption = null) {
	global $argv, $IsNonDefaultConfig, $ConfigFileName, $Configs;

	// Make the AppName a full path
	$AppName = $argv[0]; if(!strStartWith("/", $AppName)) $AppName = getcwd()."/".$AppName;
	// Get the real path so that there is no need to worry about different path via symbol-link.
	$AppName = getRealPath($AppName);
	// Add the config file suffix
	$AppName .= ($IsNonDefaultConfig?"":" --ConfigFile ".$ConfigFileName);
		
	// Install the next cron tab time
	if($ExtraOption != null) $AppName .= $ExtraOption;

	// Remove the old one -> Done by -u option above

	// add the new one
	$Result = installCrontabJob($CronTime, $AppName.' -l', $Configs["EnableCronJobEmail"]);
	if($Result == null) {
		echo "Successfully install automatic backup job".($IsNonDefaultConfig?"":" for \"".$ConfigFileName."\"").": \n\n";

		// Display result
		echo listCrontabJob()."\n";
	}
	return $Result;
}

function removeEasyBackupCrontabJop($IsExitWhenDone) {
	global $Program, $IsNonDefaultConfig, $ConfigFileName;

	// See if the old one is there
	$Citeria = $IsNonDefaultConfig
	             ?'grep -E "'.$Program.'.php" | grep -v "ConfigFile"'
	             :'grep -E "'.$Program.'.php.*--ConfigFile.*'.$ConfigFileName.'"';
	$Result = trim(shell_exec('crontab -l | '.$Citeria));

	if($Result == null) {
		if($IsExitWhenDone) {
			echo "\nThere is no automatic backup job".($IsNonDefaultConfig?"":" for \"".$ConfigFileName."\"")."\n\n";
			exit(0);
		}
	} else {
		$Citeria = $IsNonDefaultConfig
             ?'(crontab -l | grep -v -E "'.$Program.'.php "; crontab -l | grep "'.$Program.'.php.*--ConfigFile ")'
             :'(crontab -l | grep -v -E "'.$Program.'.php.*--ConfigFile.*'.$ConfigFileName.' "                  )';

		// Remove the old one
		shell_Exec($Citeria.' | crontab -');
		if($IsExitWhenDone) {
			echo "\nSuccessfully remove automatic backup job".($IsNonDefaultConfig?"":" for \"".$ConfigFileName."\"").":\n";

			// Display result
			echo listCrontabJob()."\n";
			exit(0);
		}
	}
}

/** Combine the list of excludes and includes */
function combineExcludesIncludes($E, $I) {
	// Mark the include or exclude
	for($i = count($I); --$i >= 0; ) { $P = getRealPath($I[$i]); $I[$P] = "i"; unset($I[$i]); }
	for($i = count($E); --$i >= 0; ) { $P = getRealPath($E[$i]); $E[$P] = "e"; unset($E[$i]); }
	// Merge them, key from $E will overwrite the one from $I
	$EI = array_merge($I, $E);
	// Ensure default setting for '/'
	if($EI["/"] != "i") $EI["/"] = "e";
	// Get the Sorted jey
	$KEI = array_keys($EI);
	sort($KEI);

	// Loop down from 1 to the one before the last
	$Count = count($KEI);
	for($i = 1; $i < $Count; $i++) {
		if(!isset($KEI[$i])) continue;
		$iKPath = $KEI[$i];
		// Loop up from the one above it ($i - 1) to 0
		for($j = ($i - 1); $j >= 0; --$j) {
			$jKPath = $KEI[$j];
			if($jKPath == null) continue;
			// Often used with '/' on the back
			$jKPathS = ($jKPath == "/")?"/":$jKPath."/";
			// Find the first mactch
			if(!strStartWith($jKPathS, $iKPath)) continue;
			// If this path e/i flag is the same as the one just above it, remove this one
			if($EI[$iKPath] == $EI[$jKPath]) {
				// Remove the path as its setting is the similar to the closed one above it
				unset($EI[ $iKPath]);
				unset($KEI[$i]);
			}
			
			break;
		}
	}

	// Source the array by its key (path) and reverse its order
	// Actually what we want is the inner file/folder declared first
	krsort($EI);
	$KEI = array_keys($EI);
	foreach($KEI as $K) {
		// If root is included, remove the entry as its default is always included if not specified.
		if(($K == "/") && ($EI[$K] == "i")) { unset($EI[$K]); continue; }

		// Put --exclude and --include prefix
		if($EI[$K] == "e") $EI[$K] = "--exclude '".$K."'";
		else               $EI[$K] = "--include '".$K."'";
	}
	return $EI;
}

/** Prepare rdiff-backup prefix */
function getCommandPrefixFromConfig() {
	global $Configs;
	$Parameters = $Configs["Parameters"];
	$Includes   = addToArray("--include '", $Configs["Includes"], "'");
	$Excludes   = addToArray("--exclude '", $Configs["Excludes"], "'");

	return array_merge(array('rdiff-backup'), $Parameters, $Includes, $Excludes);
}

?>
