#!/usr/bin/php
<?php require_once('EasyBackup.inc'); ?>
<?php

if(($argc != 1) && ($argc != 2))
	showError("Parameter Error", "Invalid number of parameter\n Usage:\n    ".$argv[0]." [Path to file].");

$IsTest = false;

// Processs Parameter -------------------------------------------------------------------------------------------------
if($argc > 1) {
	if(in_array("-t", $argv)) $IsTest = true;
	if(in_array("--help", $argv) || in_array("-h", $argv)) showHelpMessage();
}

// Get the Orignal location of the file and extract the file name
$OrgPath = getRealPath(getOnlyPathParameter());

// Get the target path and its snapshort
$FileToRecover = getBackupPathFromOriginal($OrgPath);
$TimeToDisplay = convertToDisplayTime(getSnapshotTimeStrings($OrgPath, true));

while(true) {
	// Let user select the time
	$TimeToRecover = showList(
						/* Title              */ "Select the snapshot to recover.",
						/* Column Header      */ array("Snapshort Time", "Action"),
						/* First Item         */ "All ...",
						/* Each Item array    */ array($TimeToDisplay[1], $TimeToDisplay[2]),
						/* Last Item          */ null,
						/* is loop backword?  */ true,
						/* Item per page      */ 10,
						/* Text for previous  */ "Newer ...",
						/* Text for next      */ "Older ...",
						/* Width and Height   */ 400, 500);
	if($TimeToRecover == null) exit(0);

	// Get the location to save
	$DisplayTitle   = 'The target file location to save the recovered file';
	$DisplayText    = $DisplayTitle.' of \"'.$OrgPath.'\"';
	$TimeToRecovers = null;
	$IsLoop         = false;

	if($TimeToRecover == "All ...") {
		$NewLocation    = $OrgPath.".recover-";
		$TimeToRecovers = array_values($TimeToDisplay[0]);
		$IsLoop         = true;
	} else {
		$NewLocation    = showGetPathAsTextForWrite($DisplayTitle, $DisplayText, $OrgPath);
		$TimeToRecovers = array($TimeToDisplay[0][$TimeToRecover]);
	}

	$IsReTry = false;
	foreach($TimeToRecovers as $Time) {
		// Prepare the command to recover
		$Location = !$IsLoop
						?$NewLocation
						:$OrgPath.'--{'.date("Y-m-j--G-i-s", strtotime($Time)).'}';
		$BackupCmd = 'rdiff-backup --force -r "'.$Time.'" "'.$FileToRecover.'" "'.$Location.'"';

		echo "=========================================================================================================\n";
		echo "Recovering $OrgPath from ".date("D\d\a\y M j Y - G:i:s", strtotime($Time))."\n";
		echo "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n";
		if($IsTest) echo "Command:\n".$BackupCmd."\n";
		else {
			// Recover the file
			echo trim(shell_exec($BackupCmd));
			if(!file_exists($Location)) {
				$IsReTry = showConfirmDialog(null,
					"It seems that you are recover the file that does not exist during the selected snapshort.\n".
					"Do you want to recover from the other snapshort?");
			}
		}
		echo "---------------------------------------------------------------------------------------------------------\n\n";
	}

	if(!$IsReTry) break;
}

echo "Finish at at ".date("Y-m[M]-d--H:i")."\n";

// Functions ----------------------------------------------------------------------------------------------------------


/** Display help message */
function showHelpMessage() {
	global $ProgramFull;
?>
<?php echo $ProgramFull."\n\n"; ?>
This command is used to recover a file from a backup. You will be ask futher for the time of the backup in which you
    want the backup file from.

-----------------------------------------------------------------------------------------------------------------------

Usage:
    <?php echo $ProgramFull; ?> <option>

    <option>
    --help, -h:
	Show this message

    -t:
	Just simulate and not actually do the recovery

<?php
	exit(0);
}

/** Get the strings of all snapshots of the given original path */
function getSnapshotTimeStrings($OrgPath, $IsErrorWhenFail) {
	global $argv;
	$BkPath = getBackupPathFromOriginal($OrgPath);
	$Result = trim(shell_exec('rdiff-backup --parsable-output -l "'.$BkPath.'"'));
	$Result = split("\n", $Result);

	if(($Result == null) || (count($Result) == 0) ||
	   ((count($Result) == 1) && (($Result[0] == null) || strEndWith(" missing", $Result[0])))) {

		if($IsErrorWhenFail) {
			showInfoGUI(null, "The file/folder ".$argv[1]." does not have any increment entries.", 800, 300);
			exit(0);
		}
		return null;
	}

	return $Result;
}

function convertToDisplayTime($Times) {
	global $argv;
	if($Times == null) return null;

	$TimeMap       = array();
	$TimeToDisplay = array();
	$Actions       = array();
	$IsMissing     = false;
	for($i = 0; $i < count($Times); $i++) {
		$Entry     = split(" ", $Times[$i]);
		$TTRecover = $Entry[0];
		$Action    = $Entry[1];
		if($Action == "missing") {
			$Action    = "Deleted";
			$IsMissing = true;
		} else if($Action == "regular") {
			if($IsMissing) $Action = "Recreated";
			else           $Action = "Changed";
			$IsMissing = false;
		}

		$TTDisplay = date("D M j Y - G:i:s", 0 + $TTRecover);
		$TimeMap[$TTDisplay] = $TTRecover;
		$TimeToDisplay[]     = $TTDisplay;
		$Actions[]           = $Action;
	}
	return array($TimeMap, $TimeToDisplay, $Actions);
}

?>
