#!/bin/sh

CURRENT=$(pwd)

EBTEMP=/tmp/EasyBackup

EBINST=/opt
EBPATH=$EBINST/EasyBackup
EBSOURCE=$EBPATH/source
EBCONFS=$EBPATH/confs
EBEMBLEMS=$EBPATH/resources/emblems

EASYBACKUP=~/.EasyBackup
NSCRIPT=~/.gnome2/nautilus-scripts/Easy\ Backup
EMBLEMS=~/.icons/hicolor/48x48/emblems/

EBDOWNLOAD=http://launchpad.net/gnomeeasybackup/release-0.1a/0.1a/+download/
EBDIST=EasyBackup-release-0.1a.tar.gz

echo "Install neccessary packages ---------------------------------------------------"
gksudo "apt-get install -y php5-cli rdiff-backup"

cd /tmp

echo "Download the file -------------------------------------------------------------"
wget $EBDOWNLOAD/$EBDIST

echo "Extract the file --------------------------------------------------------------"
tar -xzf /tmp/$EBDIST
rm -f /tmp/$EBDIST

echo "Copy the file to the installed folder -----------------------------------------"
gksudo "cp -R $EBTEMP $EBINST"

echo "Create the target folder ------------------------------------------------------"
mkdir -p $EASYBACKUP/logs

echo "Links the executables ---------------------------------------------------------"
ln -sf "$EBSOURCE/php/EasyBackup.php"  "$EASYBACKUP/EasyBackup.php"
ln -sf "$EBSOURCE/php/EasyRecover.php" "$EASYBACKUP/EasyRecover.php"
ln -sf "$EBSOURCE/php/EasyRevert.php"  "$EASYBACKUP/EasyRevert.php"

echo "Select the repository location ------------------------------------------------"
backuprepo=$(zenity --file-selection --directory --title="Select the backup repository location.")
backuprepo=$(echo $backuprepo | sed 's/\//\\\\\//g')
sed 's/^BackupDIR.*/BackupDIR = "'$backuprepo'\/"/g' "$EBCONFS/default.ini" > "$EASYBACKUP/default.ini"

echo "Create Nautilus Script --------------------------------------------------------"
mkdir -p "$NSCRIPT"
ln -sf "$EASYBACKUP/EasyBackup.php"  "$NSCRIPT/Backup"
ln -sf "$EASYBACKUP/EasyRecover.php" "$NSCRIPT/Recover"
ln -sf "$EASYBACKUP/EasyRevert.php"  "$NSCRIPT/Revert"

echo "Install emblems into Nautilus -------------------------------------------------"
mkdir -p $EMBLEMS
cp -f "$EBEMBLEMS/"*.icon "$EMBLEMS"
cp -f "$EBEMBLEMS/"*.png  "$EMBLEMS"

echo ""
echo "Finish :-D"
echo "Thank you for testing EasyBackup. Please give some feed back at http://launchpad.net/gnomeeasybackup"
echo "To uninstall type: sudo /opt/EasyBackup/uninstall.sh"

Msg="Thank you for testing EasyBackup.\n\n"
Msg=$Msg"See usages information see:\n\thttps://wiki.ubuntu.com/GnomeEasyBackup#Basic Usages\n\n"
Msg=$Msg"Please give some feed back at:\n\thttp://launchpad.net/gnomeeasybackup.\n\n"
Msg=$Msg"To uninstall type:\n\tsudo /opt/EasyBackup/uninstall.sh\n"
zenity --info --text="`printf "$Msg"`"

cd $CURRENT

