<?php require_once('Libs.inc');       ?>
<?php require_once('Lib_Zenity.inc'); ?>
<?php

echo "Staring ".$argv[0]." at ".date("Y-m[M]-d--H:i")."\n";

// ---------------------------------------------------------------------------------------------------------------------
// This file contains initial starting up for all program of EasyBackup and commonly used utilities
// ---------------------------------------------------------------------------------------------------------------------

$Program               = "EasyBackup";
$Version               = "0.1";
$ProgramFull           = $Program." v.".$Version;
$DefaultConfigFileName = "default.ini";

// Set the prefix of all dialog title
$TitlePrefix = $ProgramFull;

// Processs Parameter -------------------------------------------------------------------------------------------------
$IsForceCreate = in_array("-c", $argv);
$IsToWait      = in_array("-w", $argv);

// Ensure no other rdiff-backup is in use
EnsureNoRdiffBackupRunning();

// Confituration ----------------------------------------------------------------------------------

$ConfigFileName = $DefaultConfigFileName;
if(($argc > 1) && in_array("--ConfigFile", $argv)) {	// Select configuration
	for($i = 1;  $i < $argc; $i++) {
		if($argv[$i] != "--ConfigFile") continue;
		if($i == $argc) { echo "Invalid parameter.\n"; exit(0); }

		$ConfigFileName = $argv[$i + 1];
		break;	
	}
}
$IsNonDefaultConfig = ($ConfigFileName == $DefaultConfigFileName);

// Prepare required paramters
$UserName   = trim(shell_exec('whoami'));
$HomeDir    = "/home/".$UserName."/";
$ConfigFile = $HomeDir.".".$Program."/".$ConfigFileName;
$Configs    = parse_ini_file($ConfigFile);

// Prepare the target backup location and ensure it validity ----------------------------------------------------------
$BackupDIR = $Configs["BackupDIR"];
$BackupDIR = ($BackupDIR == null)?null:$BackupDIR.$UserName;
if($BackupDIR == null)
	showError("Configuration ERROR!!!", "No configuration file (~/".$ConfigFile.") or 'BackupDIR' entry is missing.");

if(file_exists($BackupDIR)) {
	if(!is_dir($BackupDIR) && !$IsForceCreate)
		showError("Configuration ERROR!!!", "The backup target '".$BackupDIR."' is not a directory.");
	if(!is_writable($BackupDIR) && !$Configs["BackupAsRoot"])
		showError("Configuration ERROR!!!", "The backup target '".$BackupDIR."' is not a writable.");

}

// Prepare GUI if needed
//$Display = trim(shell_exec('echo $DISPLAY'));
$Display = "export DISPLAY=:0 && ";

// Ensure the correct time zone
ensureCorrectTimeZone();

// Common Utilities -------------------------------------------------------------------------------

/** Get the parameter as path (only when one parameter is needed and as a path) */
function getOnlyPathParameter() {
	global $argc, $argv;
	// Get the Orignal location of the file and extract the file name
	// The Original location is the current path if not given as parameter.
	$OrgPath  = ($argc != 2)?getcwd():$argv[1];
	// If the given Original location is not a full path, it is a file in the current path.
	if(!strStartWith("/", $OrgPath)) $OrgPath = getcwd()."/".$OrgPath;
	if(strEndWith("/", $OrgPath))    $OrgPath = substr($OrgPath, 0, strlen($OrgPath) - 1);
	return $OrgPath;
}

/** Get the path in the backup folder related to the path of the original (in home dir) */
function getBackupPathFromOriginal($OrgPath) {
	global $BackupDIR, $HomeDir;
	$OrgPath = getRealPath($OrgPath);
	return $BackupDIR.$OrgPath;
}

/** Check if another rdiff-backup is currently running */
function EnsureNoRdiffBackupRunning() {
	global $IsToWait;
	while(trim(shell_exec('ps ax | grep "rdiff-backup" | grep -v "grep"')) != null) {
		showInfoGUI("", "Other instance of rdiff-backup is running. Please try again later.");
		if(!$IsToWait) exit(0);
	}
}

?>
