<?php

// ---------------------------------------------------------------------------------------------------------------------
// This file contains functions about Zenity GUI from Commandline.
// ---------------------------------------------------------------------------------------------------------------------

/** Show information using Zenity GUI */
function showInfoGUI($Title, $Text, $Width = -1, $Height = -1, $Timeout = -1) {
	global $TitlePrefix, $Display;
	$Dimension = (($Width == -1)?"":" --width=".$Width).(($Height == -1)?"":" --height=".$Height);
	$WaitTime  = (($Timeout == -1)?"":" --timeout=".$Timeout);
	echo "$Text\n";
	shell_exec($Display.'zenity --info --title="'.$TitlePrefix.(($Title == null)?"":" - ".$Title).'"'.$Dimension.' --text="'.$Text.'"'.$WaitTime);
}

/** Show error message using Zenity GUI */
function showError($Title, $Text, $Width = -1, $Height = -1) {
	global $TitlePrefix, $Display;
	$Text = "ERROR: ".$Text;
	echo $Text."\n";
	$Dimension = (($Width == -1)?"":" --width=".$Width).(($Height == -1)?"":" --height=".$Height);
	shell_exec($Display.'zenity --error --title="'.$TitlePrefix.(($Title == null)?"":" - ".$Title).'"'.$Dimension.' --text="'.$Text.'"');
	exit(0);
}

/** Show warning message using Zenity GUI */
function showWarning($Title, $Text, $Width = -1, $Height = -1) {
	global $TitlePrefix, $Display;
	$Text = "WARNING: ".$Text;
	echo $Text."\n";
	$Dimension = (($Width == -1)?"":" --width=".$Width).(($Height == -1)?"":" --height=".$Height);
	shell_exec($Display.'zenity --warning --title="'.$TitlePrefix.(($Title == null)?"":" - ".$Title).'"'.$Dimension.' --text="'.$Text.'"');
}

/** Ask user for a confirmation */
function showConfirmDialog($Title, $DisplayText, $Width = -1, $Height = -1) {
	global $TitlePrefix, $Display;
	$ZenityPrefix =
			$Display.'zenity --question'.
			' --title="'.$TitlePrefix.(($Title == null)?"":" - ".$Title).'"'.
			' '.(($Width == -1)?"":" --width=".$Width)." ".(($Height == -1)?"":" --height=".$Height);
	if("0" == trim(shell_exec($ZenityPrefix.' --text="'.$DisplayText.'"; echo $?')))
		return true;
	return false;
}

/** Show graphic list using Zenity GUI */
function showList($Title, $Headers, $First, $Eachs, $Last, $IsBackward, $CountPerPage, $PrevText, $NextText,
			$Width = -1, $Height = -1) {
	global $TitlePrefix, $Display;

	// Prepare prefix
	$ZenityPrefix =
			$Display.'zenity --list'.
			' --title="'.$TitlePrefix.(($Title == null)?"":" - ".$Title).'"'.
			(($Title == null)?"":(' --text="'.$Title.'"')).
			' '.(($Width  == -1)?"":" --width=".$Width).' '.(($Height == -1)?"":" --height=".$Height);

	// Prepare header
	$ColCount = count($Eachs);
	for($c = 0; $c < $ColCount; $c++)
		$ZenityPrefix = $ZenityPrefix.' --column="'.(is_null($Headers[$c])?"Choices":$Headers[$c]).'" ';

	if(is_null($First)) $First = ""; else { $First = ' "'.$First.'" '; for($c = $ColCount; --$c > 0; ) $First .= ' "" '; }
	if(is_null($Last))  $Last  = ""; else { $Last  = ' "'.$Last.'" ';  for($c = $ColCount; --$c > 0; ) $Last  .= ' "" '; }
	$Prev = (($PrevText == null)?"Previous ...":$PrevText);
	$Next = (($NextText == null)?"Next ..."    :$NextText);
	$PrevText = ' "'.$Prev.'" '; for($c = $ColCount; --$c > 0; ) $PrevText .= ' "" ';
	$NextText = ' "'.$Next.'" '; for($c = $ColCount; --$c > 0; ) $NextText .= ' "" ';

	$AllCount = count($Eachs[0]);
	$Start    = $IsBackward?($AllCount - 1):0;
	$CountPerPage--;

	while($IsBackward?($Start >= 0):($Start < $AllCount)) {
		// Prepare First and last index
		$FirstIndex = $Start;
		$LastIndex  = $Start + ($IsBackward?-$CountPerPage:$CountPerPage);
		if($LastIndex <          0) $LastIndex = 0;
		if($LastIndex >= $AllCount) $LastIndex = ($AllCount - 1);

		// Prepare each item
		$ZenityText = "";
		for($i = $FirstIndex; true; ($IsBackward?--$i:$i++)) {
			// Populate the list
			for($c = 0; $c < $ColCount; $c++) {
				//echo "i = ".$i."; c = ".$c." ZenityText = ".$ZenityText."\n";
				$ZenityText = $ZenityText." \"".$Eachs[$c][$i]."\"";
			}
			// Check for stop
			if($i == $LastIndex) break;
		}

		// Add First
		if(($IsBackward && ($FirstIndex == ($AllCount - 1))) || (!$IsBackward && ($FirstIndex == 0)))
			$ZenityText = $First.$ZenityText;

		// Add Last
		if(($IsBackward && ($LastIndex == 0)) || (!$IsBackward && ($LastIndex == ($AllCount - 1))))
			$ZenityText = $ZenityText.$Last;

		// Add Prev
		if(($IsBackward && ($FirstIndex != ($AllCount - 1))) || (!$IsBackward && ($FirstIndex != 0)))
			$ZenityText = ' '.$PrevText.' '.$ZenityText;

		// Add Next
		if(($IsBackward && ($LastIndex != 0)) || (!$IsBackward && ($LastIndex != ($AllCount - 1))))
			$ZenityText = $ZenityText.' '.$NextText.' ';

		// Add Prefix
		$ZenityText = $ZenityPrefix.$ZenityText;

		// Run it
		$ZenityResult = trim(''.shell_exec($ZenityText));
		
		if($ZenityResult == null) return null;
		if($ZenityResult == $Prev) { $Start = $Start + ($CountPerPage + 1)*($IsBackward?+1:-1); continue; }
		if($ZenityResult == $Next) { $Start = $Start + ($CountPerPage + 1)*($IsBackward?-1:+1); continue; }
		return $ZenityResult;
	}
}

/** Ask for a writable path */
function showGetPathAsTextForWrite($Title, $DisplayText, $InitialPath, $Width = -1, $Height = -1) {
	global $TitlePrefix, $Display;
	$ZenityPrefix =
			$Display.'zenity --entry'.
			' --title="'.$TitlePrefix.(($Title == null)?"":" - ".$Title).'"'.
			' '.(($Width  == -1)?"":" --width=".$Width)." ".(($Height == -1)?"":" --height=".$Height);

	$NewPath  = $InitialPath;
	while($DisplayText != null) {
		// Get the text
		$NewPath = trim(shell_exec($ZenityPrefix.' --text="'.$DisplayText.'" --entry-text="'.$NewPath.'"'));
		if($NewPath == null) exit(0);

		if(file_exists($NewPath)) {
			if(!is_writable($NewPath)) {
				$DisplayText = "The given file is not writable. Please select again.\n".$DisplayText;
				continue;
			}

			if(showConfirmDialog("The file is already exists.", "Are you sure you want to overwrite?")) break;
			continue;
		}

		break;
	}
	return $NewPath;
}

?>
