import os, calendar, datetime, Libs

# ---------------------------------------------------------------------------------------------------------------------
# This file contains functions about Crontab Job.
# ---------------------------------------------------------------------------------------------------------------------

class Cron:
	
	def listCrontabJob(Cron):
		"""Returns the list of CrontabJob currently install"""
		Command = 'crontab -l';
		Result  = "";
		for Line in os.popen(Command).readlines(): Result += Line;
		return Result.strip();
	listCrontabJob = classmethod(listCrontabJob);
	
	def installCrontabJob(Cron, Command, EnableEmailNotification):
		"""Install a Crontab Job. Command must contain no double qoute sign."""
		EmailNotification = Libs.OR(EnableEmailNotification, ' > /dev/null 2>&1', '');
		Command = '(crontab -l ; echo "'+CronTime+' '+Command+' '+EmailNotification+'") | crontab -';
		Result  = "";
		for Line in os.popen(Command).readlines(): ResultResult += Line;
		return Result.strip();
	installCrontabJob = classmethod(installCrontabJob);
	
	def getSequence(Cron, End, Step, Start = 0):
		"""Returns the string of a sequence from 0 to End by Step"""
		if(Step == 0):   return "0";
		if(Step == 1):   return "*";
		if(Step >= End): return "0";
		Seq = "" + `Start`;
		i = (Start + Step);
		while(i < End):
			Seq += "," + `i`;
			i += Step
		return Seq;
	getSequence = classmethod(getSequence);
	
	CrontabJobPattern = re.compile(r'((([0-9]+(\-[0-9]+)?(,[0-9]+(\-[0-9]+)?)*)|\*) +){4}([0-6](\-[0-6])?(,[0-6](\-[0-6])?)*|\*)');
	RepeatTimePattern = re.compile(r'(?P<T>[0-9]+)(?P<U>[mhDM]|min|hour|day|mouth)');
	
	def getCronjobTimeForEvery(Cron, Time):
		"""Returns the Cronjob time string from the  string"""
		#	A day in a week:
		#		(sun|mon|tue|wed|thu|fri|sat)
		#	Examples:
		#		sun = backup every sunday (mid night of sunday)
		#
		#	Every certail amount of time or 'every-x' time:
		#		<Step><Unit> as [0-9]+[mhDM]
		#		For Unit,
		#			m = Minute
		#			h = Hour
		#			D = Day
		#			M = Month
		#		Examples:
		#			10m = every 10 minutes
		#			12h = every 12 hours
		#			14D = every 14 days
		#
		#	A valid cronjob time:
		#		"0 0 1 * *" = For at the 1st midnight of every month
	 	
		DayOfWeek = {"sun" : 0, "mon" : 1, "tue" : 2, "wed" : 3, "thu" : 4, "fri" : 5, "sat" : 5};
		
		CrontabJobResult = Cron.CrontabJobPattern.search(Time);
		RepeatTimeResult = Cron.RepeatTimePattern.search(Time);
		
		if(DayOfWeek.has_key(Time)):
			Min  = time.localtime()[4];
			Hour = time.localtime()[3];
			return `Min` + " " + `Hour` + " * * " + `DayOfWeek.get(Time)`;
			
		elif(CrontabJobResult != None):
			return Time;
			
		elif(RepeatTimeResult != None):
			CronT = "";
			Time = int(RepeatTimeResult.group("T"));
			Unit =     RepeatTimeResult.group("U");
			
			Now = datetime.datetime.now();
			
			if(  Unit ==   "min"): Unit = "m";
			elif(Unit ==  "hour"): Unit = "h";
			elif(Unit ==   "day"): Unit = "D";
			elif(Unit == "mouth"): Unit = "M";
			
			if(  Unit == "m"): CronT = ""                + Cron.getSequence(60, Time, 0) + " * * * *";
			elif(Unit == "h"): CronT = "0 "                + Cron.getSequence(24, Time, 0) + " * * *";
			elif(Unit == "D"): CronT = "* " + Now.hour + " " + Cron.getSequence(31, Time, 1) + " * *";
			elif(Unit == "M"): CronT = "* * " + Now.day  + " " + Cron.getSequence(12, Time, 1) + " *";
			return CronT;
			
		return None;
	getCronjobTimeForEvery = classmethod(getCronjobTimeForEvery);
	
	NextTimePattern = re.compile(r'(?P<T>[0-9]+)(?P<U>[mhDWM]|mins?|hours?|days?|weeks?|mouths?)');
	
	def getCronjobTimeForNext(Cron, Time):
		"""Returns the time for the next operation in a crontab job format."""
		# The time produce by this function will ensure that the operation will be done after the specific period of time.
		# By given the crontab schedule of operating every single minute after the time period. It is very important for
		# the operation to remove the schedule and recreate the new one for the next operation.
		
		NextTimeResult = Cron.NextTimePattern.search(Time);
		if(NextTimeResult == None): return None;
		
		Time = int(NextTimeResult.group("T"));
		Unit =     NextTimeResult.group("U");
		
		Now = datetime.datetime.now();
		Restarts = [ 0,  0, 1,                              1];
		Maxs     = [59, 23, calendar.mdays[Now.month],     12];
		NextMaxs = [59, 23, calendar.mdays[Now.month + 1], 12];
		
		Next = Now;
		if(  "min"  .startswith(Unit) or (Unit ==  "m")): Next += datetime.timedelta(minutes = Time);
		elif("hour" .startswith(Unit) or (Unit ==  "h")): Next += datetime.timedelta(hours   = Time);
		elif("day"  .startswith(Unit) or (Unit ==  "D")): Next += datetime.timedelta(days    = Time);
		elif("week" .startswith(Unit) or (Unit ==  "W")): Next += datetime.timedelta(weeks   = Time);
		elif("mouth".startswith(Unit) or (Unit ==  "M")): Next += datetime.timedelta(months  = Time);
		
		Starts = [Now.minute,  Now.hour,  Now.day,  Now.month,  Now.day];
		Ends   = [Next.minute, Next.hour, Next.day, Next.month, Next.day];
		
		CronTime      = " ";
		IsFirst       = True;
		IsPrevEqual   =  (Ends[(len(Starts) - 1)] == Starts[(len(Starts) - 1)]);
		IsPrevTwoFold = ((Ends[(len(Starts) - 1)] -  Starts[(len(Starts) - 1)]) >= 2);
		
		i = (len(Starts) - 1);
		while (i >= 0):
			if(IsPrevTwoFold): CronTime = "* " + CronTime;
			else:
				Diff = Ends[i] - Starts[i];
				
				if(IsPrevEqual):
					if(Diff == 0): CronTime = "* "  + CronTime;
					else:
						if(Ends[i] == NextMaxs[i]): CronTime = `int(Ends[i])`                            + " " + CronTime;
						else:                       CronTime = `int(Ends[i])` + "-" + `int(NextMaxs[i])` + " " + CronTime;
						
						if(  (Starts[i] - 1) == Restarts[i]): CronTime = `Restarts[i]`                              + "," + CronTime;
						elif((Starts[i] - 1) >  Restarts[i]): CronTime = `Restarts[i]` + "-" + `int(Starts[i] - 1)` + "," + CronTime;
				else:
					if(IsFirst):
						if(  (Starts[i] - 1) <= Ends[i]): return None;
						else:                             CronTime = `int(Ends[i])` + "-" + `int(Starts[i] - 1)` + " " + CronTime;
					else:
						if(  Ends[i] == Restarts[i]): CronTime = "* " + CronTime;
						elif(Ends[i] == NextMaxs[i]): CronTime = `int(Ends[i])`                            + " " + CronTime;
						else:                         CronTime = `int(Ends[i])` + "-" + `int(NextMaxs[i])` + " " + CronTime;
				
				IsFirst       = False;
				IsPrevTwoFold = (not IsPrevEqual) or (Diff >= 2);
				IsPrevEqual   = (Diff == 0);
			
			i -= 1;
		
		return CronTime.strip() + " *";
	getCronjobTimeForNext = classmethod(getCronjobTimeForNext);



