import os, sys, Libs

# ----------------------------------------------------------------------------------------------------------------------
# This file contains functions about Zenity GUI from Commandline.
# ----------------------------------------------------------------------------------------------------------------------

TitlePrefix = "";
Display     = "";

def fillColumn(Text, ColCount):
	if Text == None:
		Text = "";
	else:
		Text = ' "' + Text + '" ';
		for c in range(1, ColCount): Text += ' "" ';
	return Text;


class Zenity:
	
	# Note: Width and Height does not seems to have effect when call from python
	# So I just put it here for later fix
	def showMessage(Zenity, Title, Text, Timeout = -1, MsgKind = "info", Width = -1, Height = -1):
		"""Show information using Zenity GUI"""
		global TitlePrefix, Display;
		if TitlePrefix != "": Title = TitlePrefix + " - " + Title;
		
		Command =  Display + ' zenity --' + MsgKind;
		Command += ' --title="'+ Title + '"';
		Command += ' --text="' + Text  + '"';
		if Width   != -1: Command += (" --width="   + `Width`  );
		if Height  != -1: Command += (" --height="  + `Height` );
		if Timeout != -1: Command += (" --timeout=" + `Timeout`);
		print Text;
		os.popen(Command);
	showMessage = classmethod(showMessage);
	
	def showInfo(Zenity, Title, Text, Timeout = -1, Width = -1, Height = -1):
		"""Show information using Zenity GUI"""
		Zenity.showMessage(Title, Text, Timeout, "info", Width, Height);
	showInfo = classmethod(showInfo);
	
	def showWarning(Zenity, Title, Text, Timeout = -1, Width = -1, Height = -1):
		"""Show warning message using Zenity GUI"""
		Zenity.showMessage(Title, Text, Timeout, "warning", Width, Height);
	showWarning = classmethod(showWarning);
	
	def showError(Zenity, Title, Text, Timeout = -1, Width = -1, Height = -1):
		"""Show error message using Zenity GUI"""
		Zenity.showMessage(Title, Text, Timeout, "error", Width, Height);
		sys.exit(0);
	showError = classmethod(showError);
	
	# Note: Width and Height does not seems to have effect when call from python
	# So I just put it here for later fix
	def showQuestion(Zenity, Title, Text, Width = -1, Height = -1):
		"""Ask user for a confirmation"""
		global TitlePrefix, Display;
		if TitlePrefix != "": Title = TitlePrefix + " - " + Title;
		
		Command =  Display + ' zenity --question';
		Command += ' --title="'+ Title + '"';
		Command += ' --text="' + Text  + '"';
		if Width   != -1: Command += (" --width="   + `Width`  );
		if Height  != -1: Command += (" --height="  + `Height` );
		
		return (0 == os.system(Command));
	showQuestion = classmethod(showQuestion);
	
	def showPathToWriteDialog(Zenity, Title, Path, IsDir = None, Width = -1, Height = -1):
		"""Ask user for a path to be saved on"""
		global TitlePrefix, Display;
		if TitlePrefix != "":   Title = TitlePrefix + " - " + Title;
		if IsDir       == None: IsDir = os.path.isdir(Path);
		
		Command =  Display + ' zenity --file-selection';
		Command += ' --title="'    + Title + '"';
		Command += ' --filename="' + Path  + '"';
		Command += ' --save --confirm-overwrite';
		if IsDir        : Command += " --directory";
		if Width   != -1: Command += (" --width="   + `Width`  );
		if Height  != -1: Command += (" --height="  + `Height` );
		
		FileName = "";
		for Line in os.popen(Command).readlines(): FileName += Line;
		
		return FileName.strip();
	showPathToWriteDialog = classmethod(showPathToWriteDialog);
	
	def showList(Zenity, Title, Headers, First, Eachs, Last, IsBackward, CountPerPage, PrevText, NextText, Width = -1, Height = -1):
		"""Ask user to select an item from the given list"""
		global TitlePrefix, Display;
		Text = Title;
		if TitlePrefix != "":   Title = TitlePrefix + " - " + Title;
		
		#Prepare the command prefix
		ZenityPrefix = Display + 'zenity --list';
		ZenityPrefix += ' --title="' + Title + '"';
		ZenityPrefix += ' --text="'  + Text  + '"';
		if Width   != -1: ZenityPrefix += (" --width="   + `Width`  );
		if Height  != -1: ZenityPrefix += (" --height="  + `Height` );
		
		# Prepare header
		ColCount = len(Eachs);
		for c in range(0, ColCount):
			ZenityPrefix += ' --column="' + Libs.OR((Headers[c] == None), "Choices", Headers[c]) + '" ';
		
		#Prepare First, Last, Prev and Next
		First = fillColumn(First, ColCount);
		Last  = fillColumn(Last,  ColCount);
		
		Prev = PrevText;
		Next = NextText;
		if ((Prev == None) or (len(Prev) == 0)): Prev = "Previous ...";
		if ((Next == None) or (len(Next) == 0)): Next = "Next ...";
		
		PrevText = fillColumn(Prev, ColCount);
		NextText = fillColumn(Next, ColCount);
		
		CountPerPage -= 1;
		AllCount = len(Eachs[0]);
		AllEnd   = AllCount - 1;
		Start    = 0;
		ItemStep = 1;
		PageStep = CountPerPage;
		if IsBackward:
			Start    = AllCount - 1;
			PageStep = -CountPerPage;
			ItemStep = -1;
		
		while ((IsBackward and (Start >= 0)) or ((not IsBackward) and (Start < AllCount))):
			# Prepare First and last index
			FirstIndex = Start;
			LastIndex  = Start + PageStep;
			if LastIndex <         0: LastIndex = 0;
			if LastIndex >= AllCount: LastIndex = (AllCount - 1);
			
			# Prepare each item
			Command = "";
			i = FirstIndex;
			while(True):
				for c in range(0, ColCount):
					Command += " \"" + Eachs[c][i] + "\"";
				if(i == LastIndex): break;
				i += ItemStep;
				
			# Add First
			if (IsBackward and (FirstIndex == AllEnd)) or ((not IsBackward) and (FirstIndex == 0)):
				Command = First + Command;
			# Add Last
			if (IsBackward and (LastIndex  == 0))      or ((not IsBackward) and (LastIndex  == AllEnd)):
				Command = Command + Last;
			# Add Prev
			if (IsBackward and (FirstIndex != AllEnd)) or ((not IsBackward) and (FirstIndex  != 0)):
				Command = ' ' + PrevText + ' ' + Command;
			# Add Next
			if (IsBackward and (LastIndex  != 0))      or ((not IsBackward) and (LastIndex   != AllEnd)):
				Command = Command + ' ' + NextText + ' ';
			
			# Add Prefix
			Command = ZenityPrefix + Command;
			
			# Run it
			print Command;
			ZenityResult = "";
			for Line in os.popen(Command).readlines(): ZenityResult += Line;
			ZenityResult = ZenityResult.strip();
			
			if(len(ZenityResult) == 0): return None;
			if(ZenityResult == Prev):
				Start -= (CountPerPage + 1)*ItemStep;
				
				continue;
			if(ZenityResult == Next):
				Start += (CountPerPage + 1)*ItemStep;
				continue;
			return ZenityResult;
	showList = classmethod(showList);


























